@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -125,7 +125,7 @@ void enlarge_n( std::vector< std::vector< char > > & data, const int n ) throw()
     new_data.push_back( std::vector< char >() );
     for( int col = 0; col < width; ++col )
       {
-      bool d = datarow[col];
+      const char d = datarow[col];
       for( int i = 0; i < n; ++i ) new_data.back().push_back( d );
       }
     for( int i = 1; i < n; ++i ) new_data.push_back( new_data.back() );
@@ -287,6 +287,29 @@ void Bitmap::add_rectangle( const Rectangle & re ) throw()
   }
 
 
+// Returns true if height is or can be adjusted
+//
+bool Bitmap::adjust_height() throw()
+  {
+  int row1, row2;
+
+  for( row1 = top(); row1 <= bottom(); ++row1 )
+    for( int col = left(); col <= right(); ++col )
+      if( get_bit( row1, col ) ) goto L1;
+  L1:
+  for( row2 = bottom(); row2 >= row1; --row2 )
+    for( int col = left(); col <= right(); ++col )
+      if( get_bit( row2, col ) ) goto L2;
+  L2:
+  if( row1 > top() || row2 < bottom() )
+    {
+    if( row1 <= row2 ) { top( row1 ); bottom( row2 ); }
+    else return false;
+    }
+  return true;
+  }
+
+
 // Return the total filled area of this Bitmap
 //
 int Bitmap::area() const throw()
@@ -367,6 +390,232 @@ int Bitmap::seek_bottom( const int row, const int col, const bool black ) const
   }
 
 
+bool Bitmap::escape_left( int row, int col ) const throw()
+  {
+  if( get_bit( row, col ) ) return false;
+  int u, d;
+
+  for( u = row; u > top() + 1; --u ) if( get_bit( u - 1, col ) ) break;
+  for( d = row; d < bottom() - 1; ++d ) if( get_bit( d + 1, col ) ) break;
+  while( u <= d && --col >= left() )
+    {
+    if( u > top() + 1 && !get_bit( u, col ) ) --u;
+    if( d < bottom() - 1 && !get_bit( d, col ) ) ++d;
+    while( u <= d && get_bit( u, col ) ) ++u;
+    while( u <= d && get_bit( d, col ) ) --d;
+    }
+  return ( col < left() );
+  }
+
+
+bool Bitmap::escape_top( int row, int col ) const throw()
+  {
+  if( get_bit( row, col ) ) return false;
+  int l, r;
+
+  for( l = col; l > left() + 1; --l ) if( get_bit( row, l - 1 ) ) break;
+  for( r = col; r < right() - 1; ++r ) if( get_bit( row, r + 1 ) ) break;
+  while( l <= r && --row >= top() )
+    {
+    if( l > left() + 1 && !get_bit( row, l ) ) --l;
+    if( r < right() - 1 && !get_bit( row, r ) ) ++r;
+    while( l <= r && get_bit( row, l ) ) ++l;
+    while( l <= r && get_bit( row, r ) ) --r;
+    }
+  return ( row < top() );
+  }
+
+
+bool Bitmap::escape_right( int row, int col ) const throw()
+  {
+  if( get_bit( row, col ) ) return false;
+  int u, d;
+
+  for( u = row; u > top() + 1; --u ) if( get_bit( u - 1, col ) ) break;
+  for( d = row; d < bottom() - 1; ++d ) if( get_bit( d + 1, col ) ) break;
+  while( u <= d && ++col <= right() )
+    {
+    if( u > top() + 1 && !get_bit( u, col ) ) --u;
+    if( d < bottom() - 1 && !get_bit( d, col ) ) ++d;
+    while( u <= d && get_bit( u, col ) ) ++u;
+    while( u <= d && get_bit( d, col ) ) --d;
+    }
+  return ( col > right() );
+  }
+
+
+bool Bitmap::escape_bottom( int row, int col ) const throw()
+  {
+  if( get_bit( row, col ) ) return false;
+  int l, r;
+
+  for( l = col; l > left() + 1; --l ) if( get_bit( row, l - 1 ) ) break;
+  for( r = col; r < right() - 1; ++r ) if( get_bit( row, r + 1 ) ) break;
+  while( l <= r && ++row <= bottom() )
+    {
+    if( l > left() + 1 && !get_bit( row, l ) ) --l;
+    if( r < right() - 1 && !get_bit( row, r ) ) ++r;
+    while( l <= r && get_bit( row, l ) ) ++l;
+    while( l <= r && get_bit( row, r ) ) --r;
+    }
+  return ( row > bottom() );
+  }
+
+
+int Bitmap::follow_top( int row, int col ) const throw()
+  {
+  if( !get_bit( row, col ) ) return row;
+  std::vector< char > array;
+  array.reserve( width() );
+  int c;
+
+  for( c = col; c > left() && get_bit( row, c - 1 ); --c );
+  if( c > left() ) array.resize( c - left(), false );
+  for( c = col; c < right() && get_bit( row, c + 1 ); ++c );
+  array.resize( c - left() + 1, true );
+  if( c < right() ) array.resize( width(), false );
+
+  while( --row >= top() )
+    {
+    bool alive = false;
+    for( int i = 0; i < width(); ++i ) if( array[i] )
+      { if( !get_bit( row, left() + i ) ) array[i] = false;
+      else alive = true; }
+    if( !alive ) break;
+
+    for( int i = 1; i < width(); ++i )
+      if( array[i-1] && !array[i] &&
+          get_bit( row, left() + i ) ) array[i] = true;
+    for( int i = width() - 2; i >= 0; --i )
+      if( array[i+1] && !array[i] &&
+          get_bit( row, left() + i ) ) array[i] = true;
+    }
+  return row + 1;
+  }
+
+
+int Bitmap::follow_bottom( int row, int col ) const throw()
+  {
+  if( !get_bit( row, col ) ) return row;
+  std::vector< char > array;
+  array.reserve( width() );
+  int c;
+
+  for( c = col; c > left() && get_bit( row, c - 1 ); --c );
+  if( c > left() ) array.resize( c - left(), false );
+  for( c = col; c < right() && get_bit( row, c + 1 ); ++c );
+  array.resize( c - left() + 1, true );
+  if( c < right() ) array.resize( width(), false );
+
+  while( ++row <= bottom() )
+    {
+    bool alive = false;
+    for( int i = 0; i < width(); ++i ) if( array[i] )
+      { if( !get_bit( row, left() + i ) ) array[i] = false;
+      else alive = true; }
+    if( !alive ) break;
+
+    for( int i = 1; i < width(); ++i )
+      if( array[i-1] && !array[i] &&
+          get_bit( row, left() + i ) ) array[i] = true;
+    for( int i = width() - 2; i >= 0; --i )
+      if( array[i+1] && !array[i] &&
+          get_bit( row, left() + i ) ) array[i] = true;
+    }
+  return row - 1;
+  }
+
+
+// Looks for an inverted-U-shaped curve near the top, then tests which of
+// the vertical bars goes deeper
+//
+bool Bitmap::top_hook( int *hdiff ) const throw()
+  {
+  int row, lcol = 0, rcol = 0, black_section = 0, wmax = 0;
+
+  for( row = top() + 1; row < vcenter(); ++row )
+    {
+    int l = -1, r = -2;
+    bool prev_black = false;
+    black_section = 0;
+    for( int col = left(); col <= right(); ++col )
+      {
+      bool black = get_bit( row, col );
+      if( black )
+        {
+        if( !prev_black && ++black_section == 2 ) rcol = col;
+        r = col; if( l < 0 ) l = col;
+        }
+      else if( prev_black && black_section == 1 ) lcol = col - 1;
+      prev_black = black;
+      }
+    r = r - l + 1;
+    if( 10 * r <= 9 * wmax ) return false;
+    if( r > wmax ) wmax = r ;
+    if( black_section >= 2 ) break;
+    }
+
+  if( black_section != 2 ) return false;
+  if( escape_top( row, lcol + 1 ) ) return false;
+  int lrow = follow_bottom( row, lcol ), rrow = follow_bottom( row, rcol );
+  if( lrow <= row || rrow <= row ) return false;
+  if( hdiff ) *hdiff = lrow - rrow;
+  return true;
+  }
+
+
+// Looks for an U-shaped curve near the bottom, then tests which of
+// the vertical bars is taller
+//
+bool Bitmap::bottom_hook( int *hdiff ) const throw()
+  {
+  int row, lcol = 0, rcol = 0, black_section = 0, wmax = 0;
+
+  for( row = bottom(); row > vpos( 80 ); --row )
+    {
+    int l, r;
+    for( l = left(); l <= right(); ++l ) if( get_bit( row, l ) ) break;
+    for( r = right(); r > l; --r ) if( get_bit( row, r ) ) break;
+    const int w = r - l + 1;
+    if( w > wmax ) wmax = w;
+    if( 4 * w >= width() )
+      {
+      int i;
+      for( i = l + 1; i < r; ++i ) if( !get_bit( row, i ) ) break;
+      if( i >= r ) break;
+      }
+    }
+
+  if( row > vpos( 80 ) ) while( --row > vcenter() )
+    {
+    int l = -1, r = -2;
+    bool prev_black = false;
+    black_section = 0;
+    for( int col = left(); col <= right(); ++col )
+      {
+      bool black = get_bit( row, col );
+      if( black )
+        {
+        if( !prev_black && ++black_section == 2 ) rcol = col;
+        r = col; if( l < 0 ) l = col;
+        }
+      else if( prev_black && black_section == 1 ) lcol = col - 1;
+      prev_black = black;
+      }
+    const int w = r - l + 1;
+    if( black_section > 2 || 10 * w <= 8 * wmax ) break;
+    if( w > wmax ) wmax = w;
+    if( black_section == 2 && rcol - lcol >= 2 )
+      {
+      if( escape_bottom( row, lcol + 1 ) ) break;
+      if( hdiff ) *hdiff = follow_top( row, lcol ) - follow_top( row, rcol );
+      return true;
+      }
+    }
+  return false;
+  }
+
+
 bool Bitmap::scale( int n, const Rational & th ) throw( Bitmap::Error )
   {
   if( n <= -2 )
