@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #include <algorithm>
@@ -22,22 +22,6 @@
 #include "common.h"
 
 
-void Ocrad::internal_error( const char * msg ) throw()
-  {
-  std::fprintf( stderr, "ocrad: internal error: %s.\n", msg );
-  exit( 3 );
-  }
-
-
-bool Ocrad::similar( int a, int b, int percent_dif, int abs_dif ) throw()
-  {
-  int difference = std::abs( a - b );
-  if( percent_dif > 0 && difference <= abs_dif ) return true;
-  int max_abs = std::max( std::abs(a), std::abs(b) );
-  return ( difference * 100 <= max_abs * percent_dif );
-  }
-
-
 namespace {
 
 const int charsets = 3;
@@ -50,7 +34,7 @@ const char * const charset_name[charsets] =
 
 struct T_entry
   {
-  char * name;
+  const char * name;
   Transformation::Type type;
   };
 
@@ -70,6 +54,15 @@ const T_entry T_table[] =
 } // end namespace
 
 
+bool Ocrad::similar( int a, int b, int percent_dif, int abs_dif ) throw()
+  {
+  int difference = std::abs( a - b );
+  if( percent_dif > 0 && difference <= abs_dif ) return true;
+  int max_abs = std::max( std::abs(a), std::abs(b) );
+  return ( difference * 100 <= max_abs * percent_dif );
+  }
+
+
 bool Charset::enable( const char * name ) throw()
   {
   for( int i = 0; i < charsets; ++i )
@@ -93,10 +86,10 @@ bool Charset::only( Value cset ) const throw()
   }
 
 
-void Charset::show_error( const char * arg ) const throw()
+void Charset::show_error( const char * program_name, const char * arg ) const throw()
   {
   if( arg && std::strcmp( arg, "help" ) )
-    std::fprintf( stderr,"ocrad: bad charset '%s'\n", arg );
+    std::fprintf( stderr,"%s: bad charset '%s'\n", program_name, arg );
   std::fputs( "Valid charset names are:", stderr );
   for( int i = 0; i < charsets; ++i )
     std::fprintf( stderr, "  %s", charset_name[i] );
@@ -113,14 +106,14 @@ bool Transformation::set( const char * name ) throw()
   }
 
 
-void Transformation::show_error( const char * arg ) const throw()
+void Transformation::show_error( const char * program_name, const char * arg ) const throw()
   {
   if( arg && std::strcmp( arg, "help" ) )
-    std::fprintf( stderr,"ocrad: bad bitmap trasformation '%s'\n", arg );
+    std::fprintf( stderr,"%s: bad bitmap trasformation '%s'\n", program_name, arg );
   std::fputs( "Valid transformation names are:", stderr );
   for( int i = 0; T_table[i].name != 0; ++i )
     std::fprintf( stderr, "  %s", T_table[i].name );
-  std::fputs( "\n", stderr );
+  std::fputs( "\nRotations are made counter-clockwise.\n", stderr );
   }
 
 
