@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,11 +16,14 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-namespace Ocrad
-  {
-  void internal_error( const char * msg ) throw() __attribute__ ((noreturn));
-  bool similar( int a, int b, int percent_dif, int abs_dif = 1 ) throw();
-  }
+namespace Ocrad {
+
+extern bool verbose;
+
+void internal_error( const char * msg ) throw() __attribute__ ((noreturn));
+bool similar( int a, int b, int percent_dif, int abs_dif = 1 ) throw();
+
+} // end namespace Ocrad
 
 
 class Charset
@@ -63,11 +66,10 @@ struct Control
   FILE *outfile, *exportfile;
   int debug_level;
   char filetype;
-  bool verbose;
 
   Control() throw()
     : format( byte ), outfile( stdout ), exportfile( 0 ),
-      debug_level( 0 ), filetype( 0 ), verbose( false ) {}
+      debug_level( 0 ), filetype( '4' ) {}
 
   bool set_format( const char * name ) throw();
   };
