@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include <algorithm>
 #include <cstdio>
 #include <vector>
+
 #include "common.h"
 #include "rectangle.h"
 #include "ucs.h"
@@ -56,7 +57,7 @@ int Features::test_49ARegpq( const Rectangle & charbox ) const throw()
       if( row2 < b.vpos( 75 ) ) return 'g';
       if( row2 < b.bottom() ) return 'e';
       }
-    if( bp.minima() == 1 ) return 'p';
+    return 'p';
     }
 
   else if( b.escape_left( row, col ) )
@@ -82,7 +83,8 @@ int Features::test_49ARegpq( const Rectangle & charbox ) const throw()
     int row2 = b.seek_bottom( row, col );
     if( row2 < b.bottom() && rp.increasing( ( ( row + row2 ) / 2 ) - b.top() ) )
       return 'g';
-    if( bp.minima() == 1 ) return 'q';
+    if( bp.minima() == 1 )
+      { if( h.height() >= charbox.height() ) return 'Q'; else return 'q'; }
     }
   return 0;
   }
@@ -154,7 +156,7 @@ int Features::test_6abd( const Charset & charset ) const throw()
   const Bitmap & h = b.hole( 0 );
 
   if( 3 * h.width() < b.width() &&
-      ( bp.minima( b.height() / 4 ) != 1 || tp.isctip() ) ) return 0;
+      ( bp.minima( b.height() / 4 ) != 1 || tp.minima( h.vcenter() - b.top() ) != 1 ) ) return 0;
 
   int col = h.hcenter();
   int row = b.seek_top( h.top(), col, false ) - 1;
