@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,47 +29,42 @@
 #include <string>
 #include <vector>
 #include <getopt.h>
+
 #include "common.h"
+#include "rational.h"
 #include "rectangle.h"
-#include "bitmap.h"
+#include "page_image.h"
 #include "textpage.h"
 
 
 namespace {
 
-// Date of this version: 2005-06-07
+// Date of this version: 2006-02-15
 
 const char * invocation_name = 0;
 const char * const Program_name    = "GNU Ocrad";
 const char * const program_name    = "ocrad";
-const char * const program_version = "0.12";
-const char * const program_year    = "2005";
+const char * const program_version = "0.14";
+const char * const program_year    = "2006";
 
 struct Input_control
   {
   Transformation transformation;
-  int rindex, layout_level, scale, threshold;
+  int rindex, layout_level, scale;
+  Rational threshold, ltrb[4];
   char specialtype;
-  bool copy, invert;
+  bool copy, crop, invert;
 
   Input_control() throw()
     : rindex( -1 ), layout_level( 0 ), scale( 0 ), threshold( -1 ),
-      specialtype( 0 ), copy( false ), invert( false )
-    {}
-  bool set_format( const char * name ) throw();
+      specialtype( 0 ), copy( false ), crop( false ), invert( false ) {}
+
+  bool read_crop_rectangle( const char * ptr ) throw();
+  bool set_threshold( const char * ptr ) throw();
   };
 
 
-void show_version() throw()
-  {
-  std::printf( "%s version %s\n", Program_name, program_version );
-  std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "This program is free software; you may redistribute it under the terms of\n" );
-  std::printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
-  }
-
-
-void show_error( const char * msg, int errcode = 0, bool help = false ) throw()
+void show_error( const char * msg, const int errcode = 0, const bool help = false ) throw()
   {
   if( msg && msg[0] != 0 )
     {
@@ -82,10 +77,42 @@ void show_error( const char * msg, int errcode = 0, bool help = false ) throw()
   }
 
 
-void show_help( bool verbose ) throw()
+bool Input_control::read_crop_rectangle( const char * ptr ) throw()
+  {
+  int c = ltrb[0].parse( ptr );				// left
+  if( c && ltrb[0] >= 0 && ptr[c] == ',' )
+    {
+    int i = c + 1;
+    c = ltrb[1].parse( &ptr[i] );			// top
+    if( c && ltrb[1] >= 0 && ptr[i+c] == ',' )
+      {
+      i += c + 1; c = ltrb[2].parse( &ptr[i] );		// right
+      if( c && ltrb[2] > 0 && ptr[i+c] == ',' )
+        {
+        i += c + 1; c = ltrb[3].parse( &ptr[i] );	// bottom
+        if( c && ltrb[3] > 0 ) { crop = true; return true; }
+        }
+      }
+    }
+  show_error( "invalid crop rectangle", 0, true );
+  return false;
+  }
+
+
+bool Input_control::set_threshold( const char * ptr ) throw()
+  {
+  Rational tmp;
+  if( tmp.parse( ptr ) && tmp >= 0 && tmp <= 1 )
+    { threshold = tmp; return true; }
+  show_error( "threshold out of limits (0.0 - 1.0)", 0, true );
+  return false;
+  }
+
+
+void show_help() throw()
   {
   std::printf( "%s - Optical Character Recognition program.\n", Program_name );
-  std::printf( "Reads pbm file(s), or standard input, and sends text to standard output.\n" );
+  std::printf( "Reads pnm file(s), or standard input, and sends text to standard output.\n" );
   std::printf( "\nUsage: %s [options] [files]\n", invocation_name );
   std::printf( "Options:\n" );
   std::printf( "  -h, --help               display this help and exit\n" );
@@ -98,22 +125,32 @@ void show_help( bool verbose ) throw()
   std::printf( "  -i, --invert             invert image levels (white on black)\n" );
   std::printf( "  -l, --layout=<n>         layout analysis, 0=none, 1=column, 2=full\n" );
   std::printf( "  -o <file>                place the output into <file>\n" );
+  std::printf( "  -p, --crop=<l,t,r,b>     crop input image by given rectangle\n" );
   std::printf( "  -s, --scale=[-]<n>       scale input image by [1/]<n>\n" );
   std::printf( "  -t, --transform=<name>   try `--transform=help' for a list of names\n" );
+  std::printf( "  -T, --threshold=<n%%>     threshold for binarization (0-100%%)\n" );
   std::printf( "  -v, --verbose            be verbose\n" );
   std::printf( "  -x <file>                export OCR Results File to <file>\n" );
-  if( verbose )
+  if( Ocrad::verbose )
     {
-    std::printf( "  -1, -4                   pbm output file type (debug)\n" );
+    std::printf( "  -1..6                    pnm output file type (debug)\n" );
     std::printf( "  -C, --copy               'copy' input to output (debug)\n" );
     std::printf( "  -D, --debug=<level>      (0-100) output intermediate data (debug)\n" );
     std::printf( "  -S <type>                make a 'special file' (debug)\n" );
-    std::printf( "  -T <threshold>           set threshold for -s option (debug)\n" );
     }
   std::printf( "\nReport bugs to bug-ocrad@gnu.org\n" );
   }
 
 
+void show_version() throw()
+  {
+  std::printf( "%s version %s\n", Program_name, program_version );
+  std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
+  std::printf( "This program is free software; you may redistribute it under the terms of\n" );
+  std::printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
+  }
+
+
 const char * my_basename( const char * filename ) throw()
   {
   const char * c = filename;
@@ -126,28 +163,31 @@ int process_file( FILE *infile, const char * infile_name,
                   const Input_control & input_control,
                   const Control & control ) throw()
   {
+  if( Ocrad::verbose )
+    std::fprintf( stderr, "processing file `%s'\n", infile_name );
   try
     {
-    std::vector< Rectangle > rv;
-    Bitmap page_image( infile, input_control.invert );
+    Page_image page_image( infile, input_control.threshold, input_control.invert );
 
-    page_image.transform( input_control.transformation );
-    page_image.scale( input_control.scale, input_control.threshold );
-    page_image.analyse_layout( rv, input_control.layout_level );
-    if( control.verbose ) std::fprintf( stderr, "number of text blocks %d\n",
-                                        rv.size() );
-
-    if( input_control.rindex >= (int)rv.size() )
+    if( input_control.crop &&
+        !page_image.crop( input_control.ltrb, input_control.threshold ) )
       {
-      std::fprintf( stderr,"This page has only %d text block(s)\n", rv.size() );
+      if( Ocrad::verbose )
+        std::fprintf( stderr, "file `%s' totally cropped out\n", infile_name );
       return 1;
       }
-    if( input_control.rindex >= 0 )
-      {
-      Rectangle re = rv[input_control.rindex];
-      rv.clear(); rv.push_back( re );
-      }
+    page_image.transform( input_control.transformation );
+    page_image.scale( input_control.scale );
+    page_image.analyse_layout( input_control.layout_level );
+    if( Ocrad::verbose )
+      std::fprintf( stderr, "number of text blocks = %d\n", page_image.zones() );
+    if( input_control.threshold < 0 ) page_image.adapt_thresholds();
 
+    if( input_control.rindex >= page_image.zones() )
+      {
+      std::fprintf( stderr,"This page has only %d text block(s)\n", page_image.zones() );
+      return 1;
+      }
     if( input_control.specialtype != 0 )
       {
       if( control.outfile )
@@ -163,26 +203,28 @@ int process_file( FILE *infile, const char * infile_name,
       {
       if( control.outfile )
         {
-        if( input_control.layout_level == 0 && rv.size() == 1 )
+        if( input_control.layout_level == 0 && page_image.zones() == 1 )
           page_image.save( control.outfile, control.filetype );
-        else for( unsigned int c = 0; c < rv.size(); ++c )
-          {
-          Bitmap bitmap( page_image, rv[c] );
-          bitmap.save( control.outfile, control.filetype );
-          }
+        else
+          for( int c = 0; c < page_image.zones(); ++c )
+            if( input_control.rindex < 0 || input_control.rindex == c )
+              page_image.save( control.outfile, control.filetype, c );
         }
       return 0;
       }
 
-    Textpage textpage( page_image, rv, my_basename( infile_name ), control );
+    Textpage textpage( page_image, my_basename( infile_name ), control );
     }
-  catch( Bitmap::Error e ) { show_error( e.s ); return 2; }
+  catch( Page_image::Error e ) { show_error( e.s ); return 2; }
+  if( Ocrad::verbose ) std::fprintf( stderr, "\n" );
   return 0;
   }
 
 } // end namespace
 
 
+bool Ocrad::verbose = false;
+
 void Ocrad::internal_error( const char * msg ) throw()
   {
   char buf[80];
@@ -192,10 +234,10 @@ void Ocrad::internal_error( const char * msg ) throw()
   }
 
 
-// 'infile' contains the scanned image (in pbm format) to be converted
+// 'infile' contains the scanned image (in pnm format) to be converted
 // to text.
 // 'outfile' is the destination for the text version of the scanned
-// image. (or for a pbm file if debugging).
+// image. (or for a pnm file if debugging).
 // 'exportfile' is the Ocr Results File.
 //
 int main( int argc, char * argv[] ) throw()
@@ -212,38 +254,44 @@ int main( int argc, char * argv[] ) throw()
     {
     static const struct option long_options[] =
       {
-      {"append", no_argument, 0, 'a'},
-      {"block", required_argument, 0, 'b'},
-      {"charset", required_argument, 0, 'c'},
-      {"copy", no_argument, 0, 'C'},
-      {"debug", required_argument, 0, 'D'},
-      {"force", no_argument, 0, 'f'},
-      {"format", required_argument, 0, 'F'},
-      {"help", no_argument, 0, 'h'},
-      {"invert", no_argument, 0, 'i'},
-      {"layout", required_argument, 0, 'l'},
-      {"scale", required_argument, 0, 's'},
+      {"append",          no_argument, 0, 'a'},
+      {"block",     required_argument, 0, 'b'},
+      {"charset",   required_argument, 0, 'c'},
+      {"copy",            no_argument, 0, 'C'},
+      {"crop",      required_argument, 0, 'p'},
+      {"debug",     required_argument, 0, 'D'},
+      {"force",           no_argument, 0, 'f'},
+      {"format",    required_argument, 0, 'F'},
+      {"help",            no_argument, 0, 'h'},
+      {"invert",          no_argument, 0, 'i'},
+      {"layout",    required_argument, 0, 'l'},
+      {"scale",     required_argument, 0, 's'},
       {"transform", required_argument, 0, 't'},
-      {"verbose", no_argument, 0, 'v'},
-      {"version", no_argument, 0, 'V'},
+      {"threshold", required_argument, 0, 'T'},
+      {"verbose",         no_argument, 0, 'v'},
+      {"version",         no_argument, 0, 'V'},
       {0, 0, 0, 0}
       };
 
-    int c = getopt_long( argc, argv, "14CD:F:S:T:Vab:c:fhil:o:s:t:vx:",
+    int c = getopt_long( argc, argv, "123456CD:F:S:T:Vab:c:fhil:o:p:s:t:vx:",
                          long_options, 0 );
     if( c == -1 ) break;		// all options processed
 
     switch( c )
       {
       case '1':
-      case '4': control.filetype = c; break;
+      case '2':
+      case '3':
+      case '4':
+      case '5':
+      case '6': control.filetype = c; break;
       case 'C': input_control.copy = true; break;
       case 'D': control.debug_level = std::strtol( optarg, 0, 0 ); break;
       case 'F': if( !control.set_format( optarg ) )
                   { show_error( "bad output format", 0, true ); return 1; }
                 break;
       case 'S': input_control.specialtype = optarg[0]; break;
-      case 'T': input_control.threshold = std::strtol( optarg, 0, 0 ); break;
+      case 'T': if( !input_control.set_threshold( optarg ) ) return 1; break;
       case 'V':	show_version(); return 0;
       case 'a': append = true; break;
       case 'b': input_control.rindex = std::strtol( optarg, 0, 0 ) - 1; break;
@@ -251,16 +299,17 @@ int main( int argc, char * argv[] ) throw()
                   { control.charset.show_error( program_name, optarg ); return 1; }
                 break;
       case 'f': force = true; break;
-      case 'h': show_help( control.verbose ); return 0;
+      case 'h': show_help(); return 0;
       case 'i': input_control.invert = true; break;
       case 'l': input_control.layout_level = std::strtol( optarg, 0, 0 ); break;
       case 'o':	outfile_name = optarg; break;
+      case 'p': if( !input_control.read_crop_rectangle( optarg ) ) return 1; break;
       case 's': input_control.scale = std::strtol( optarg, 0, 0 ); break;
       case 't': if( !input_control.transformation.set( optarg ) )
                   { input_control.transformation.show_error( program_name, optarg );
                   return 1; }
                 break;
-      case 'v': control.verbose = true; break;
+      case 'v': Ocrad::verbose = true; break;
       case 'x':	exportfile_name = optarg; break;
       case '?': show_error( 0, 0, true ); return 1;	// bad option
       default: return 1;
