@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,70 +27,67 @@
 
 
 Profile::Profile( const Bitmap & b, Type t ) throw()
-  : _bitmap( &b ), type( t ), valid( false ),
-    valid_max( false ), valid_min( false ), valid_mean( false ),
-    valid_isconcave( false ), valid_isconvex( false ),
-    valid_isflat( false ), valid_isflats( false ),
-    valid_ispit( false ), valid_istpit( false ), valid_isupit( false ),
-    valid_isvpit( false ), valid_istip( false ) {}
+  : _bitmap( &b ), type( t ),
+    _limit( -1 ), _max( -1 ), _min( -1 ), _mean( -1 ),
+    _isconcave( -1 ), _isconvex( -1 ), _isflat( -1 ), _isflats( -1 ),
+    _ispit( -1 ), _istpit( -1 ), _isupit( -1 ), _isvpit( -1 ), _istip( -1 ) {}
 
 
 void Profile::initialize() throw()
   {
   const Bitmap & b = *_bitmap;
 
-  valid = true;
   switch( type )
     {
     case left :
       data.resize( b.height() ); _limit = b.width();
-      for( int i = 0; i < samples(); ++i )
+      for( int row = b.top(); row <= b.bottom(); ++row )
         {
-        int j = b.left(), row = b.top() + i;
+        int j = b.left();
         while( j <= b.right() && !b.get_bit( row, j ) ) ++j;
-        data[i] = j - b.left();
+        data[row-b.top()] = j - b.left();
         } break;
     case top :
       data.resize( b.width() ); _limit = b.height();
-      for( int i = 0; i < samples(); ++i )
+      for( int col = b.left(); col <= b.right(); ++col )
         {
-        int j = b.top(), col = b.left() + i;
+        int j = b.top();
         while( j <= b.bottom() && !b.get_bit( j, col ) ) ++j;
-        data[i] = j - b.top();
+        data[col-b.left()] = j - b.top();
         } break;
     case right :
       data.resize( b.height() ); _limit = b.width();
-      for( int i = 0; i < samples(); ++i )
+      for( int row = b.top(); row <= b.bottom(); ++row )
         {
-        int j = b.right(), row = b.top() + i;
+        int j = b.right();
         while( j >= b.left() && !b.get_bit( row, j ) ) --j;
-        data[i] = b.right() - j;
+        data[row-b.top()] = b.right() - j;
         } break;
     case bottom :
       data.resize( b.width() ); _limit = b.height();
-      for( int i = 0; i < samples(); ++i )
+      for( int col = b.left(); col <= b.right(); ++col )
         {
-        int j = b.bottom(), col = b.left() + i;
+        int j = b.bottom();
         while( j >= b.top() && !b.get_bit( j, col ) ) --j;
-        data[i] = b.bottom() - j;
+        data[col-b.left()] = b.bottom() - j;
         } break;
     case height :
       data.resize( b.width() ); _limit = b.height();
-      for( int i = 0; i < samples(); ++i )
+      for( int col = b.left(); col <= b.right(); ++col )
         {
-        int u = b.top(), d = b.bottom(), col = b.left() + i;
+        int u = b.top(), d = b.bottom();
         while( u <= d && !b.get_bit( u, col ) ) ++u;
         while( u <= d && !b.get_bit( d, col ) ) --d;
-        data[i] = ( u <= d ) ? d - u + 1 : 0;
+        data[col-b.left()] = d - u + 1;
         } break;
     case width :
       data.resize( b.height() ); _limit = b.width();
-      for( int i = 0; i < samples(); ++i )
+      for( int row = b.top(); row <= b.bottom(); ++row )
         {
-        int l = b.left(), r = b.right(), row = b.top() + i;
+        int l = b.left(), r = b.right();
         while( l <= r && !b.get_bit( row, l ) ) ++l;
         while( l <= r && !b.get_bit( row, r ) ) --r;
-        data[i] = ( l <= r ) ? r - l + 1 : 0;
+        data[row-b.top()] = r - l + 1;
         } break;
     }
   }
@@ -98,9 +95,9 @@ void Profile::initialize() throw()
 
 int Profile::mean() throw()
   {
-  if( !valid_mean )
+  if( _mean < 0 )
     {
-    valid_mean = true; if( !valid ) initialize();
+    if( _limit < 0 ) initialize();
     _mean = 0;
     for( int i = 0; i < samples(); ++i ) _mean += data[i];
     if( samples() > 1 ) _mean /= samples();
@@ -111,9 +108,9 @@ int Profile::mean() throw()
 
 int Profile::max() throw()
   {
-  if( !valid_max )
+  if( _max < 0 )
     {
-    valid_max = true; if( !valid ) initialize();
+    if( _limit < 0 ) initialize();
     _max = data[0];
     for( int i = 1; i < samples(); ++i ) if( data[i] > _max ) _max = data[i];
     }
@@ -123,7 +120,7 @@ int Profile::max() throw()
 
 int Profile::max( int l, int r ) throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   if( r < 0 ) r = samples() - 1;
   int m = 0;
   for( int i = l; i <= r; ++i ) if( data[i] > m ) m = data[i];
@@ -133,9 +130,9 @@ int Profile::max( int l, int r ) throw()
 
 int Profile::min() throw()
   {
-  if( !valid_min )
+  if( _min < 0 )
     {
-    valid_min = true; if( !valid ) initialize();
+    if( _limit < 0 ) initialize();
     _min = data[0];
     for( int i = 1; i < samples(); ++i ) if( data[i] < _min ) _min = data[i];
     }
@@ -145,7 +142,7 @@ int Profile::min() throw()
 
 int Profile::min( int l, int r ) throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   if( r < 0 ) r = samples() - 1;
   int m = _limit;
   for( int i = l; i <= r; ++i ) if( data[i] < m ) m = data[i];
@@ -155,7 +152,7 @@ int Profile::min( int l, int r ) throw()
 
 int Profile::operator[]( int i ) throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   if( i < 0 ) i = 0;
   else if( i >= samples() ) i = samples() - 1;
   return data[i];
@@ -164,7 +161,7 @@ int Profile::operator[]( int i ) throw()
 
 int Profile::area( int l, int r ) throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   if( r < 0 ) r = samples() - 1;
   int area = 0;
   for( int i = l; i <= r; ++i ) area += data[i];
@@ -174,7 +171,7 @@ int Profile::area( int l, int r ) throw()
 
 bool Profile::increasing( int i ) throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   if( i <= 0 || i > samples() - 2 || data[samples()-1] - data[i] < 2 )
     return false;
   while( ++i < samples() ) if( data[i] < data[i-1] ) return false;
@@ -184,19 +181,21 @@ bool Profile::increasing( int i ) throw()
 
 bool Profile::decreasing( int i ) throw()
   {
-  if( !valid ) initialize();
-  if( i < 0 || i >= samples() - 2 || data[samples()-1] - data[i] > -2 )
+  if( _limit < 0 ) initialize();
+  const int noise = ( std::min( samples(), _limit ) / 20 ) + 1;
+  if( i < 0 || samples() - i <= 2 * noise ||
+      data[samples()-noise] - data[i] > -( noise + 1 ) )
     return false;
-  while( ++i < samples() - 1 ) if( data[i] > data[i-1] ) return false;
+  while( ++i < samples() - noise ) if( data[i] > data[i-1] ) return false;
   return true;
   }
 
 
 bool Profile::isconcave() throw()
   {
-  if( !valid_isconcave )
+  if( _isconcave < 0 )
     {
-    valid_isconcave = true; _isconcave = false; if( !valid ) initialize();
+    _isconcave = false; if( _limit < 0 ) initialize();
     if( samples() < 5 ) return _isconcave;
     int dmax = -1, l = 0, r = 0;
 
@@ -221,9 +220,9 @@ bool Profile::isconcave() throw()
 
 bool Profile::isconvex() throw()
   {
-  if( !valid_isconvex )
+  if( _isconvex < 0 )
     {
-    valid_isconvex = true; _isconvex = false; if( !valid ) initialize();
+    _isconvex = false; if( _limit < 0 ) initialize();
     if( samples() < 9 || _limit < 5 ) return _isconvex;
     int min = _limit, min_begin = 0, min_end = 0;
     int lmin = _limit, rmax = -_limit, l = 0, r = 0;
@@ -270,15 +269,15 @@ bool Profile::isconvex() throw()
 
 bool Profile::isflat() throw()
   {
-  if( !valid_isflat )
+  if( _isflat < 0 )
     {
-    valid_isflat = true; _isflat = false; if( !valid ) initialize();
+    _isflat = false; if( _limit < 0 ) initialize();
     if( samples() < 15 ) return _isflat;
     int mn = data[samples()/2], mx = mn;
 
     for( int i = 1; i < samples() - 1; ++i )
       { int d = data[i]; if( d < mn ) mn = d; else if( d > mx ) mx = d; }
-    _isflat = ( mx - mn <= 1 + ( samples() / 30 ) );
+    _isflat = (bool)( mx - mn <= 1 + ( samples() / 30 ) );
     }
   return _isflat;
   }
@@ -286,9 +285,9 @@ bool Profile::isflat() throw()
 
 bool Profile::isflats() throw()
   {
-  if( !valid_isflats )
+  if( _isflats < 0 )
     {
-    valid_isflats = true; _isflats = false; if( !valid ) initialize();
+    _isflats = false; if( _limit < 0 ) initialize();
     if( samples() < 15 ) return _isflats;
     const int s1 = std::max( pos( 15 ), 3 );
     const int s2 = std::min( pos( 85 ), samples() - 4 );
@@ -301,7 +300,7 @@ bool Profile::isflats() throw()
     for( int i = s1 + 1; i < s2; ++i )
       { int d = data[i]; if( d < mn ) mn = d; else if( d > mx ) mx = d; }
     for( int i = s2; i < samples() - 1; ++i ) if( data[i] > mx ) mx = data[i];
-    _isflats = ( mx - mn <= 1 + ( samples() / 30 ) );
+    _isflats = (bool)( mx - mn <= 1 + ( samples() / 30 ) );
     }
   return _isflats;
   }
@@ -309,9 +308,9 @@ bool Profile::isflats() throw()
 
 bool Profile::ispit() throw()
   {
-  if( !valid_ispit )
+  if( _ispit < 0 )
     {
-    valid_ispit = true; _ispit = false; if( !valid ) initialize();
+    _ispit = false; if( _limit < 0 ) initialize();
     if( samples() < 5 ) return _ispit;
     const int nsize = samples() / 30;
     for( int i = 0; i <= nsize; ++i )
@@ -345,9 +344,9 @@ bool Profile::ispit() throw()
   }
 
 
-bool Profile::iscpit( int cpos ) throw()
+bool Profile::iscpit( const int cpos ) throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   if( samples() < 5 || cpos < 25 || cpos > 75 ) return false;
   int th = ( mean() < 2 ) ? 2 : mean();
   int imin = -1, mid = ( ( samples() - 1 ) * cpos ) / 100;
@@ -371,7 +370,7 @@ bool Profile::iscpit( int cpos ) throw()
 
 bool Profile::islpit() throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   if( samples() < 5 ) return false;
   const int nsize = samples() / 30;
   if( data[0] < nsize + 2 ) return false;
@@ -391,9 +390,9 @@ bool Profile::islpit() throw()
 
 bool Profile::istpit() throw()
   {
-  if( !valid_istpit )
+  if( _istpit < 0 )
     {
-    valid_istpit = true; if( !valid ) initialize();
+    if( _limit < 0 ) initialize();
     if( _limit < 5 || samples() < 5 || !ispit() )
       { _istpit = false; return _istpit; }
 
@@ -401,7 +400,7 @@ bool Profile::istpit() throw()
     int l = -1, r = 0;
     for( int i = 0; i < samples(); ++i )
       if( data[i] <= noise ) { r = i; if( l < 0 ) l = i; }
-    _istpit = ( l > 0 && 4 * ( r - l + 1 ) < samples() );
+    _istpit = (bool)( l > 0 && 4 * ( r - l + 1 ) < samples() );
     }
   return _istpit;
   }
@@ -409,9 +408,9 @@ bool Profile::istpit() throw()
 
 bool Profile::isupit() throw()
   {
-  if( !valid_isupit )
+  if( _isupit < 0 )
     {
-    valid_isupit = true; _isupit = false; if( !valid ) initialize();
+    _isupit = false; if( _limit < 0 ) initialize();
     if( samples() < 5 ) return _isupit;
 
     int th = ( mean() < 2 && range() > 2 ) ? 2 : mean();
@@ -438,7 +437,7 @@ bool Profile::isupit() throw()
       }
     if( ucount > 1 ) umean /= ucount;
     if( lcount > 1 ) lmean /= lcount;
-    _isupit = ( status == 2 && umean - lmean > range() / 2 );
+    _isupit = (bool)( status == 2 && umean - lmean > range() / 2 );
     }
   return _isupit;
   }
@@ -446,9 +445,9 @@ bool Profile::isupit() throw()
 
 bool Profile::isvpit() throw()
   {
-  if( !valid_isvpit )
+  if( _isvpit < 0 )
     {
-    valid_isvpit = true; if( !valid ) initialize();
+    if( _limit < 0 ) initialize();
     if( _limit < 5 || samples() < 5 || !ispit() )
       { _isvpit = false; return _isvpit; }
 
@@ -462,7 +461,7 @@ bool Profile::isvpit() throw()
         if( data[i] <= noise ) { rn = i; if( ln < 0 ) ln = i; }
         }
     const int wl = rl - ll + 1, wn = rn - ln + 1;
-    _isvpit = ( ln > 0 && 2 * wl <= samples() && wl - wn <= 2 * ( level - noise ) );
+    _isvpit = (bool)( ln > 0 && 2 * wl <= samples() + 1 && wl - wn <= 2 * ( level - noise ) );
     }
   return _isvpit;
   }
@@ -470,9 +469,9 @@ bool Profile::isvpit() throw()
 
 bool Profile::istip() throw()
   {
-  if( !valid_istip )
+  if( _istip < 0 )
     {
-    _istip = false; valid_istip = true; if( !valid ) initialize();
+    _istip = false; if( _limit < 0 ) initialize();
     if( samples() < 5 ) return _istip;
 
     int th = ( mean() < 2 && range() > 2 ) ? 2 : mean(); if( th < 2 ) ++th;
@@ -496,18 +495,19 @@ bool Profile::istip() throw()
         case 2: if( data[i] <= th ) status = 3; break;
         case 3: if( data[i] > th + 1 ) return _istip;
         }
-    _istip = ( status >= 2 );
+    _istip = (bool)( status >= 2 );
     }
   return _istip;
   }
 
 
-bool Profile::isctip( int cpos ) throw()
+bool Profile::isctip( const int cpos ) throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   if( samples() < 5 || cpos < 25 || cpos > 75 ) return false;
-  int th = std::max( 2, mean() );
-  int imax = -1, mid = ( ( samples() - 1 ) * cpos ) / 100;
+  const int mid = ( ( samples() - 1 ) * cpos ) / 100;
+  int th = std::max( 2, std::min( mean(), _limit / 3 ) );
+  int imax = -1;
 
   for( int i = 0; i < samples() / 4; ++i )
     {
@@ -522,8 +522,8 @@ bool Profile::isctip( int cpos ) throw()
       if( data[mid+i] > th ) { imax = mid + i; break; }
       if( data[mid-i-1] > th ) { imax = mid - i - 1; break; }
       }
-    if( imax < 0 ) return false;
     }
+  if( imax < 0 ) return false;
 
   for( int i = imax + 1; i < samples(); ++i )
     if( data[i] < th )
@@ -537,7 +537,7 @@ bool Profile::isctip( int cpos ) throw()
 
 int Profile::imaximum() throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   const int margin = ( samples() / 30 ) + 1;
   int mbegin = 0, mend, mvalue = 0;
 
@@ -553,7 +553,7 @@ int Profile::imaximum() throw()
 
 int Profile::iminimum( int m, int th ) throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   if( samples() < 1 ) return false;
   if( th < 2 ) th = ( mean() < 2 ) ? 2 : mean();
   int minima = 0, status = 0;
@@ -578,7 +578,7 @@ int Profile::iminimum( int m, int th ) throw()
 
 int Profile::minima( int th ) throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   if( !samples() ) return false;
   if( th < 1 ) th = ( mean() < 2 ) ? 2 : mean();
   const int noise = _limit / 40;
@@ -599,7 +599,7 @@ int Profile::minima( int th ) throw()
 
 bool Profile::straight( int * _dy ) throw()
   {
-  if( !valid ) initialize();
+  if( _limit < 0 ) initialize();
   if( samples() < 5 ) return false;
 
   const int xl = ( samples() / 30 ) + 1, yl = ( data[xl] + data[xl+1] ) / 2 ;
