@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,6 +48,10 @@ public:
   int vpos( const int p ) const throw()
     { return _top + ( ( ( _bottom - _top ) * p ) / 100 ); }
 
+  bool operator==( const Rectangle & re ) const throw()
+    { return ( _left == re._left && _top == re._top && _right == re._right && _bottom == re._bottom ); }
+  bool operator!=( const Rectangle & re ) const throw() { return !( *this == re ); }
+
   bool includes( const Rectangle & re ) const throw();
   bool includes( const int row, const int col ) const throw();
   bool strictly_includes( const Rectangle & re ) const throw();
@@ -60,8 +64,8 @@ public:
   bool v_includes( const int row ) const throw();
   bool h_overlaps( const Rectangle & re ) const throw();
   bool v_overlaps( const Rectangle & re ) const throw();
-  bool is_hcentered_in( const Rectangle & re ) const throw();
-  bool is_vcentered_in( const Rectangle & re ) const throw();
+  bool is_hcentred_in( const Rectangle & re ) const throw();
+  bool is_vcentred_in( const Rectangle & re ) const throw();
   bool h_precedes( const Rectangle & re ) const throw();
   bool v_precedes( const Rectangle & re ) const throw();
 
