@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,13 +18,15 @@
 
 #include <cstdio>
 #include <vector>
+
 #include "common.h"
+#include "rational.h"
 #include "rectangle.h"
-#include "vrhomboid.h"
 #include "track.h"
 #include "bitmap.h"
 #include "block.h"
 #include "character.h"
+#include "page_image.h"
 #include "textline.h"
 #include "textblock.h"
 
@@ -81,6 +83,72 @@ void join_characters( std::vector< Textline * > & tlpv ) throw()
     }
   }
 
+
+// Return the character position >= first preceding a big gap or eol.
+//
+int find_big_gap( const Textline & line, const int first, const int space_width_limit ) throw()
+  {
+  int i = first;
+  while( i + 1 < line.characters() )
+    {
+    Character & c1 = line.character( i );
+    Character & c2 = line.character( i + 1 );
+    const int gap = c2.left() - c1.right() - 1;
+    if( gap > space_width_limit ) break; else ++i;
+    }
+  return i;
+  }
+
+
+// Insert spaces between characters.
+//
+void insert_spaces( std::vector< Textline * > & tlpv ) throw()
+  {
+  for( unsigned int current_line = 0; current_line < tlpv.size(); ++current_line )
+    {
+    Textline & line1 = *tlpv[current_line];
+    const Rational mw = line1.mean_width();
+    if( mw < 2 ) continue;
+    const int space_width_limit = ( 3 * mw ).trunc();
+    int first = 0;
+    while( first + 1 < line1.characters() )
+      {
+      int last = find_big_gap( line1, first, space_width_limit );
+      const Rational mg = line1.mean_gap_width( first, last );
+      if( first < last && mg >= 0 )
+        {
+        int spaces = 0, nospaces = 0, spsum = 0, nospsum = 0;
+        for( int i = first ; i < last; ++i )
+          {
+          Character & c1 = line1.character( i );
+          Character & c2 = line1.character( i + 1 );
+          const int gap = c2.left() - c1.right() - 1;
+          if( gap >= mw.trunc() || gap > 3 * mg ||
+              ( 5 * gap > 2 * mw && gap > 2 * mg ) ||
+              ( 3 * c1.width() > 2 * mw && 3 * c2.width() > 2 * mw && 2 * gap > mw && 5 * gap > 8 * mg ) )
+            { ++spaces; spsum += gap;
+              if( line1.insert_space( i + 1 ) ) { ++i; ++last; } }
+          else { ++nospaces; nospsum += gap; }
+          }
+        if( spaces && nospaces )
+          {
+          const Rational th = ( Rational( 3 * spsum, spaces ) + Rational( nospsum, nospaces ) ) / 4;
+          for( int i = first ; i < last; ++i )
+            {
+            Character & c1 = line1.character( i );
+            Character & c2 = line1.character( i + 1 );
+            const int gap = c2.left() - c1.right() - 1;
+            if( gap > th && line1.insert_space( i + 1 ) ) { ++i; ++last; }
+            }
+          }
+        }
+      if( ++last < line1.characters() && line1.insert_space( last, true ) )
+        ++last;
+      first = last;
+      }
+    }
+  }
+
 } // end namespace
 
 
@@ -202,10 +270,23 @@ Textblock::Textblock( const Rectangle & r, const Charset & charset,
         ++current_line;
       if( current_line >= textlines() )
         { --current_line; delete &b; continue; }
-      if( b.height() > 2 * tlpv[current_line]->mean_height() )
-        { if( b.hcenter() < tlpv[current_line]->character(0).left() )
-            tlpv[current_line]->big_initial( new Character( &b ) ); }
-      else tlpv[current_line]->shift_characterp( new Character( &b ) );
+      Textline & l = *tlpv[current_line];
+      if( b.height() <= 3 * l.mean_height() &&
+          ( b.height() <= 2 * l.mean_height() ||
+            ( l.big_initial() && l.big_initial()->left() < b.left() ) ||
+            ( !l.big_initial() && l.character(0).left() < b.left() ) ) )
+        l.shift_characterp( new Character( &b ) );
+      else		// possible big initial
+        {
+        if( l.big_initial() && l.big_initial()->height() <= 2 * l.mean_height() )
+          {
+          l.shift_characterp( new Character( *l.big_initial() ) );
+          l.big_initial( 0 );
+          }
+        if( ( !l.big_initial() || l.big_initial()->left() > b.left() ) &&
+            l.character(0).left() > b.hcenter() )
+          l.big_initial( new Character( &b ) );
+        }
       }
     }
 
@@ -284,37 +365,34 @@ Textblock::Textblock( const Rectangle & r, const Charset & charset,
       }
     }
 
-  // Fifth pass. Add spaces between characters.
-  for( int current_line = 0; current_line < textlines(); ++current_line )
+  // Remove leading and trailing noise characters.
+  for( int i = 0; i < textlines(); ++i )
     {
-    Textline & line1 = *tlpv[current_line];
-    const int mw = line1.mean_width();
-    const int space_width_limit = line1.space_width_limit( mw );
-    const int mg = line1.mean_gap_width( space_width_limit );
-    if( mw < 2 || mg < 0 ) continue;
-    int spaces = 0, nospaces = 0, spsum = 0, nospsum = 0;
-    for( int i = 1 ; i < line1.characters(); ++i )
+    Textline & l = *tlpv[i];
+    if( !l.big_initial() && l.characters() > 2 )
       {
-      Character & c1 = line1.character( i - 1 );
-      Character & c2 = line1.character( i );
-      const int gap = c2.left() - c1.right() - 1;
-      if( gap > space_width_limit )
-        { if( line1.insert_space( i, true ) ) ++i; }
-      else if( gap >= mw || ( 3 * gap > mw && gap > 2 * mg ) )
-        { ++spaces; spsum += gap; if( line1.insert_space( i ) ) ++i; }
-      else { ++nospaces; nospsum += gap; }
+      const Character & c0 = l.character( 0 );
+      const Character & c1 = l.character( 1 );
+      const Character & c2 = l.character( 2 );
+      if( c0.blocks() == 1 &&
+          4 * c0.size() < c1.size() && c1.left() - c0.right() > 2 * l.height() &&
+          4 * c0.size() < c2.size() && c2.left() - c1.right() < l.height() )
+        l.delete_character( 0 );
       }
-    if( !spaces || !nospaces ) continue;
-    const int th = ( 2 * ( spsum / spaces ) ) + ( nospsum / nospaces ) + 3;
-    for( int i = 1 ; i < line1.characters(); ++i )
+    if( l.characters() > 2 )
       {
-      Character & c1 = line1.character( i - 1 );
-      Character & c2 = line1.character( i );
-      const int gap = c2.left() - c1.right() - 1;
-      if( 3 * gap > th ) { if( line1.insert_space( i ) ) ++i; }
+      const Character & c0 = l.character( l.characters() - 1 );
+      const Character & c1 = l.character( l.characters() - 2 );
+      const Character & c2 = l.character( l.characters() - 3 );
+      if( c0.blocks() == 1 &&
+          4 * c0.size() < c1.size() && c0.left() - c1.right() > 2 * l.height() &&
+          4 * c0.size() < c2.size() && c1.left() - c2.right() < l.height() )
+        l.delete_character( l.characters() - 1 );
       }
     }
 
+  insert_spaces( tlpv );
+
   // Recognize characters.
   for( int i = 0; i < textlines(); ++i )
     {
@@ -351,7 +429,6 @@ Textblock::Textblock( const Rectangle & r, const Charset & charset,
         for( int i = 1; i < newlines; ++i ) insert_line( tlpv, ++current_line );
         }
     }
-
   }
 
 
@@ -414,13 +491,13 @@ void Textblock::xprint( const Control & control ) const throw()
   }
 
 
-void Textblock::cmark( Bitmap & bitmap ) const throw()
+void Textblock::cmark( Page_image & page_image ) const throw()
   {
-  for( int i = 0; i < textlines(); ++i ) tlpv[i]->cmark( bitmap );
+  for( int i = 0; i < textlines(); ++i ) tlpv[i]->cmark( page_image );
   }
 
 
-void Textblock::lmark( Bitmap & bitmap ) const throw()
+void Textblock::lmark( Page_image & page_image ) const throw()
   {
-  for( int i = 0; i < textlines(); ++i ) bitmap.draw_track( *tlpv[i] );
+  for( int i = 0; i < textlines(); ++i ) page_image.draw_track( *tlpv[i] );
   }
